'use strict';

const Redis = require('ioredis');
const util = require('util');
const layouts = require('log4js').layouts;


function logstashRedis(config, layout) {
    const redis = config.redis ? new Redis(config.redis) : new Redis();
    layout = layout || layouts.dummyLayout;
    if (!config.fields) {
        config.fields = {};
    }

    return function log(loggingEvent) {
        let fields = {};
        for (let k in config.fields) {
            fields[k] = config.fields[k];
        }
        fields.level = loggingEvent.level.levelStr;
        fields.category = loggingEvent.categoryName;

        if (loggingEvent.data.length > 1) {
            const evtData = loggingEvent.data[loggingEvent.data.length - 1];
            if (util.isObject(evtData)) {
                for (let k in evtData) {
                    fields[k] = evtData[k];
                }
            }
        }

        const logObject = {
            '@version': '1',
            '@timestamp': (new Date(loggingEvent.startTime)).toISOString(),
            "type": config.logType ? config.logType : config.category,
            "message": layout(loggingEvent),
            "fields": fields
        };
        const logString = JSON.stringify(logObject);
        redis.rpush(config.key, logString, function (err, result) {
            if (err && !config.ignoreError) {
                console.error("log4js-redis-logstash - Error: %s", util.inspect(err))
            }
        });
    };
}


function configure(config) {
    let layout;
    if (config.layout) {
        layout = layouts.layout(config.layout.type, config.layout);
    }
    return logstashRedis(config, layout);
}


exports.appender = logstashRedis;
exports.configure = configure;